@@ -14,12 +14,12 @@
 #include <sycl/atomic.hpp>         // for atomic
 #include <sycl/builtins.hpp>       // for isinf, isnan, signbit
 #include <sycl/detail/array.hpp>   // for array
-#include <sycl/detail/cg.hpp>      // for stream_impl
 #include <sycl/detail/defines.hpp> // for __SYCL_SPECIAL_CLASS, __S...
 #include <sycl/detail/defines_elementary.hpp> // for __SYCL2020_DEPRECATED
 #include <sycl/detail/export.hpp>             // for __SYCL_EXPORT
 #include <sycl/detail/item_base.hpp>          // for id, range
 #include <sycl/detail/owner_less_base.hpp>    // for OwnerLessBase
+#include <sycl/ext/oneapi/bfloat16.hpp>       // for bfloat16
 #include <sycl/group.hpp>                     // for group
 #include <sycl/h_item.hpp>                    // for h_item
 #include <sycl/half_type.hpp>                 // for half, operator-, operator<
@@ -43,6 +43,8 @@ inline namespace _V1 {
 
 namespace detail {
 
+class stream_impl;
+
 using FmtFlags = unsigned int;
 
 // Mapping from stream_manipulator to FmtFlags. Each manipulator corresponds
@@ -83,10 +85,8 @@ constexpr size_t MAX_ARRAY_SIZE =
 constexpr unsigned FLUSH_BUF_OFFSET_SIZE = 2;
 
 template <class F, class T = void>
-using EnableIfFP = typename std::enable_if_t<std::is_same_v<F, float> ||
-                                                 std::is_same_v<F, double> ||
-                                                 std::is_same_v<F, half>,
-                                             T>;
+using EnableIfFP = typename std::enable_if_t<
+    detail::check_type_in_v<F, float, double, half, ext::oneapi::bfloat16>, T>;
 
 using GlobalBufAccessorT = accessor<char, 1, sycl::access::mode::read_write,
                                     sycl::access::target::device>;
@@ -334,9 +334,9 @@ checkForInfNan(char *Buf, T Val) {
     return append(Buf, "nan");
 
   // Extract the sign from the bits
-  const uint16_t Sign = reinterpret_cast<uint16_t &>(Val) & 0x8000;
+  const uint16_t Sign = sycl::bit_cast<uint16_t>(Val) & 0x8000;
   // Extract the exponent from the bits
-  const uint16_t Exp16 = (reinterpret_cast<uint16_t &>(Val) & 0x7c00) >> 10;
+  const uint16_t Exp16 = (sycl::bit_cast<uint16_t>(Val) & 0x7c00) >> 10;
 
   if (Exp16 == 0x1f) {
     if (Sign)
@@ -346,6 +346,26 @@ checkForInfNan(char *Buf, T Val) {
   return 0;
 }
 
+template <typename T>
+inline typename std::enable_if_t<std::is_same_v<T, ext::oneapi::bfloat16>,
+                                 unsigned>
+checkForInfNan(char *Buf, T Val) {
+  if (Val != Val)
+    return append(Buf, "nan");
+
+  // Extract the sign from the bits
+  const uint16_t Sign = sycl::bit_cast<uint16_t>(Val) & 0x8000;
+  // Extract the exponent from the bits
+  const uint16_t Exp16 = (sycl::bit_cast<uint16_t>(Val) & 0x7f80) >> 7;
+
+  if (Exp16 == 0x7f) {
+    if (Sign)
+      return append(Buf, "-inf");
+    return append(Buf, "inf");
+  }
+  return 0;
+}
+
 template <typename T>
 EnableIfFP<T, unsigned> floatingPointToDecStr(T AbsVal, char *Digits,
                                               int Precision, bool IsSci) {
@@ -894,9 +914,13 @@ public:
 
   bool operator!=(const stream &LHS) const;
 
-  template <typename propertyT> bool has_property() const noexcept;
+  template <typename propertyT> bool has_property() const noexcept {
+    return getPropList().template has_property<propertyT>();
+  }
 
-  template <typename propertyT> propertyT get_property() const;
+  template <typename propertyT> propertyT get_property() const {
+    return getPropList().template get_property<propertyT>();
+  }
 
 private:
 #ifdef __SYCL_DEVICE_ONLY__
@@ -904,7 +928,8 @@ private:
 #else
   std::shared_ptr<detail::stream_impl> impl;
   template <class Obj>
-  friend decltype(Obj::impl) detail::getSyclObjImpl(const Obj &SyclObject);
+  friend const decltype(Obj::impl) &
+  detail::getSyclObjImpl(const Obj &SyclObject);
 #endif
 
   // NOTE: Some members are required for reconstructing the stream, but are not
@@ -1023,8 +1048,6 @@ private:
     WIOffset = GlobalOffset[1].fetch_add(FlushBufferSize);
 
     // Initialize flush subbuffer's offset for each work item on device.
-    // Initialization on host device is performed via submition of additional
-    // host task.
     SetFlushBufOffset(GlobalFlushBuf, WIOffset, 0);
   }
 
@@ -1033,9 +1056,6 @@ private:
     // necessary if user hasn't yet flushed data on its own and kernel execution
     // is finished
     // NOTE: A call to this function will be generated by compiler
-    // NOTE: In the current implementation user should explicitly flush data on
-    // the host device. Data is not flushed automatically after kernel execution
-    // because of the missing feature in scheduler.
     flushBuffer(GlobalOffset, GlobalBuf, GlobalFlushBuf, WIOffset);
   }
 #endif
@@ -1053,6 +1073,8 @@ private:
   friend const stream &operator<<(const stream &, const float &);
   friend const stream &operator<<(const stream &, const double &);
   friend const stream &operator<<(const stream &, const half &);
+  friend const stream &operator<<(const stream &,
+                                  const ext::oneapi::bfloat16 &);
 
   friend const stream &operator<<(const stream &, const stream_manipulator);
 
@@ -1092,6 +1114,8 @@ private:
   template <int Dimensions>
   friend const stream &operator<<(const stream &Out,
                                   const h_item<Dimensions> &RHS);
+
+  const property_list &getPropList() const;
 };
 
 #if (!defined(_HAS_STD_BYTE) || _HAS_STD_BYTE != 0)
@@ -1159,6 +1183,14 @@ inline const stream &operator<<(const stream &Out, const half &RHS) {
   return Out;
 }
 
+inline const stream &operator<<(const stream &Out,
+                                const ext::oneapi::bfloat16 &RHS) {
+  detail::writeFloatingPoint<ext::oneapi::bfloat16>(
+      Out.GlobalFlushBuf, Out.FlushBufferSize, Out.WIOffset, Out.get_flags(),
+      Out.get_width(), Out.get_precision(), RHS);
+  return Out;
+}
+
 // Pointer
 
 template <typename ElementType, access::address_space Space,
